package compbio.engine;

import java.io.File;
import java.io.FileFilter;
import java.io.IOException;
import java.util.Date;
import java.util.HashMap;
import java.util.Map;

import org.apache.log4j.Level;
import org.apache.log4j.Logger;

import compbio.engine.client.ConfExecutable;
import compbio.engine.conf.PropertyHelperManager;
import compbio.metadata.JobStatus;
import compbio.util.FileUtil;
import compbio.util.PropertyHelper;

/**
 * Number of runs of each WS = number of folders with name
 * 
 * Number of successful runs = all runs with no result file
 * 
 * Per period of time = limit per file creating time Runtime (avg/max) =
 * 
 * started time - finished time Task & result size = result.size
 * 
 * Abandoned runs - not collected runs
 * 
 * Cancelled runs - cancelled
 * 
 * Cluster vs local runs
 * 
 * Reasons for failure = look in the err out?
 * 
 * 
 * Metadata required:
 * 
 * work directory for local and cluster tasks = from Helper or cmd parameter. WS
 * names - enumeration. Status file names and content.
 * 
 * @author pvtroshin
 * 
 */
public class ExecutionStatCollector {

	private static final Logger log = Logger
			.getLogger(ExecutionStatCollector.class);

	static PropertyHelper ph = PropertyHelperManager.getPropertyHelper();

	static String getClusterJobDir() {
		String clusterdir = ph.getProperty("cluster.tmp.directory");
		if (clusterdir != null) {
			clusterdir.trim();
		}
		return clusterdir;
	}

	static String getLocalJobDir() {
		String locdir = ph.getProperty("local.tmp.directory");
		if (locdir != null) {
			locdir.trim();
		}
		return locdir;
	}

	/**
	 * 
	 * @param args
	 */
	public static void main(String[] args) {

		String workDir = PropertyHelperManager.getLocalPath()
				+ getLocalJobDir().trim();
		System.out.println(workDir);
		File[] files = FileUtil.getFiles("H:/www-jws2/job_dir/jobsout",
				directories);
		for (File file : files) {
			JobDirectory jd = new JobDirectory(file);
			System.out.println(jd.getJobReport());
		}
	}

	static FileFilter directories = new FileFilter() {
		@Override
		public boolean accept(File pathname) {
			return pathname.isDirectory();
		}
	};

	static class JobDirectory {
		static final int UNDEFINED = -1;

		File jobdir;
		Map<String, File> files = new HashMap<String, File>();

		public JobDirectory(File directory) {
			this.jobdir = directory;
			for (File f : jobdir.listFiles()) {
				files.put(f.getName(), f);
			}
		}

		public boolean hasStatus(JobStatus status) {
			return files.containsKey(status.toString());
		}

		long getStartTime() {
			long starttime = UNDEFINED;
			try {
				File startfile = files.get(JobStatus.STARTED.toString());
				if (startfile == null) {
					startfile = files.get(JobStatus.SUBMITTED.toString());
				}
				if (startfile != null) {
					String start = FileUtil.readFileToString(startfile);
					starttime = Long.parseLong(start.trim());
				}
			} catch (IOException e) {
				log.log(Level.WARN,
						"Cannot parse start time: " + e.getMessage(), e);
			} catch (NumberFormatException e) {
				log.log(Level.WARN,
						"Cannot parse start time: " + e.getMessage(), e);
			}
			return starttime;
		}
		long getFinishedTime() {
			long ftime = UNDEFINED;
			try {
				File finished = files.get(JobStatus.FINISHED.toString());
				if (finished != null) {
					String start = FileUtil.readFileToString(finished);
					ftime = Long.parseLong(start.trim());
					// System.out.println("f " + ftime);
				}
			} catch (IOException e) {
				log.log(Level.WARN,
						"Cannot parse finished time: " + e.getMessage(), e);
			} catch (NumberFormatException e) {
				log.log(Level.WARN,
						"Cannot parse finished time: " + e.getMessage(), e);
			}
			return ftime;
		}

		public int getRuntime() {
			return (int) (getFinishedTime() - getStartTime());
		}

		String getWSName() {
			String name = jobdir.getName().split("#")[0];
			if (name.startsWith(ConfExecutable.CLUSTER_TASK_ID_PREFIX)) {
				assert ConfExecutable.CLUSTER_TASK_ID_PREFIX.length() == 1;
				name = name.substring(1);
			}
			return name;
		}

		// Mafft, Muscle, Tcoffee, Clustal task:fasta.in result:fasta.out
		// Probcons task:fasta.in result:alignment.out
		/*
		 * TODO replace with Universal names for WS!
		 */
		long getResultSize() {
			String name = getWSName();
			File f = null;
			if (name.equalsIgnoreCase("Probcons")) {
				f = files.get("alignment.out");
			}
			f = files.get("fasta.out");
			if (f != null) {
				return f.length();
			}
			return UNDEFINED;
		}

		long getInputSize() {
			File input = files.get("fasta.in");
			if (input != null) {
				return input.length();
			}
			return UNDEFINED;
		}

		String getJobReport() {
			String report = "JOB: " + jobdir.getName() + "\n";
			if (getStartTime() != UNDEFINED) {
				report += "Started " + new Date(getStartTime()) + "\n";
			}
			if (getFinishedTime() != UNDEFINED) {
				report += "Finished " + new Date(getFinishedTime()) + "\n";
			}
			if (getStartTime() != UNDEFINED && getFinishedTime() != UNDEFINED) {
				report += "Runtime " + getRuntime() + "\n";
			}
			report += "Input size " + getInputSize() + "\n";
			report += "Result size " + getResultSize() + "\n";
			return report;
		}
	}
}
